@@ -5,20 +5,43 @@ require Carp;
 
 ######################################################################
 
-use vars qw( %Defaults );
+######################################################################
 
-# Debugging flag activates warns throughout the code
-$Defaults{ debug } = 0;
-
-sub defaults {
-  return %Defaults
+use Class::MixinFactory -hasafactory;
+for my $factory ( (__PACKAGE__)->mixin_factory ) {
+  $factory->base_class( "Text::MicroMason::Base" );
+  $factory->mixin_prefix( "Text::MicroMason" );
 }
 
 ######################################################################
 
-# $mason = $class->new( %options );
-# $clone = $object->new( %options );
-sub new {
+######################################################################
+
+sub new { 
+  my $callee = shift;
+  my ( @traits, @attribs );
+  while ( scalar @_ ) {
+    if (  $_[0] =~ /^\-(\w+)$/ ) {
+      push @traits, $1;
+      shift;
+    } else {
+      push @attribs, splice(@_, 0, 2);
+    }
+  }
+  if ( scalar @traits ) {
+    die("Adding moxins to an existing class not supported yet!") 
+	unless ( $callee eq __PACKAGE__ );
+    $callee->class( @traits )->create( @attribs ) 
+  } else {
+    $callee->create( @attribs ) 
+  }
+}
+
+######################################################################
+
+# $mason = $class->create( %options );
+# $clone = $object->create( %options );
+sub create {
   my $referent = shift;
   if ( ! ref $referent ) {
     bless { $referent->defaults(), @_ }, $referent;
@@ -27,6 +50,111 @@ sub new {
   }
 }
 
+sub defaults {
+  return ()
+}
+
+######################################################################
+
+######################################################################
+
+# $code_ref = $mason->compile( text => $template, %options );
+# $code_ref = $mason->compile( lines => \@template, %options );
+# $code_ref = $mason->compile( file => $filename, %options );
+# $code_ref = $mason->compile( handle => $filehandle, %options );
+sub compile {
+  my ( $self, $src_type, $src_data, %options ) = @_;
+
+  ($self, $src_type, $src_data) = $self->prepare($src_type, $src_data,%options);
+  
+  my $code = $self->interpret( $src_type, $src_data );
+  
+  $self->eval_sub( $code ) 
+    or $self->croak_msg( "MicroMason compilation failed: $@\n$code\n" )
+}
+
+######################################################################
+
+# $result = $mason->execute( code => $subref, @arguments );
+# $result = $mason->execute( $src_type, $src_data, @arguments );
+# $result = $mason->execute( $src_type, $src_data, \%options, @arguments );
+sub execute {
+  my $self = shift;
+  my $sub = ( $_[0] eq 'code' ) ? do { shift; shift } : 
+	$self->compile( shift, shift, ref($_[0]) ? %{ shift() } : () )
+    or $self->croak_msg("Couldn't compile: $@");
+  &$sub( @_ );
+}
+
+######################################################################
+
+######################################################################
+
+# ($self, $src_type, $src_data) = $self->prepare($src_type, $src_data, %options)
+sub prepare {
+  my ( $self, $src_type, $src_data, %options ) = @_;
+  $self = $self->create( %options ) if ( scalar keys %options );
+  return ( $self, $src_type, $src_data );
+}
+
+######################################################################
+
+# $perl_code = $mason->interpret( $src_type, $src_data );
+sub interpret {
+  my ( $self, $src_type, $src_data ) = @_;
+  my $template = $self->read( $src_type, $src_data );
+  my @tokens = $self->lex( $template );
+  my $code = $self->assemble( @tokens );
+  return $code;
+}
+
+######################################################################
+
+# $code_ref = $mason->eval_sub( $perl_code );
+sub eval_sub {
+  my ( $m, $code ) = @_;
+  package Text::MicroMason::Commands; 
+  eval( $code )
+}
+
+######################################################################
+
+######################################################################
+
+# $template = $mason->read( $src_type, $src_data );
+sub read {
+  my ( $self, $src_type, $src_data ) = @_;
+
+  my $src_method = "read_$src_type";
+  $self->can($src_method) 
+      or $self->croak_msg("Unsupported source type '$src_type'");
+  $self->$src_method( $src_data );
+}
+
+# $template = $mason->read_text( $template );
+sub read_text {
+  ref($_[1]) ? $$_[1] : $_[1];
+}
+
+# $contents = $mason->read_file( $filename );
+sub read_file {
+  my ( $self, $file ) = @_;
+  local *FILE;
+  open FILE, "$file" or $self->croak_msg("MicroMason can't open $file: $!");
+  local $/ = undef;
+  local $_ = <FILE>;
+  close FILE or $self->croak_msg("MicroMason can't close $file: $!");;
+  return $_;
+}
+
+# $contents = $mason->read_handle( $filehandle );
+sub read_handle {
+  my ( $self, $handle ) = @_;
+  my $fh = (ref $handle eq 'GLOB') ? $handle : $$handle;
+  local $/ = undef;
+  <$fh>
+}
+
 ######################################################################
 
 # @token_pairs = $mason->lex( $template );
@@ -43,8 +171,6 @@ sub lex {
 	  && die "Couldn't find applicable parsing rule at '$1'\n";
     push @tokens, @parsed;
   }
-  $self->debug_msg( "Source:", length($_), $_ ); 
-  $self->debug_msg( "Tokens:", @tokens ); 
   return @tokens;
 }
 
@@ -55,66 +181,82 @@ sub lex_token {
 
 ######################################################################
 
+######################################################################
+
 # Text elements used for subroutine assembly
-use vars qw( %Assembler );
-
 sub assembler_rules {
-  my $self = shift;
-  return %Assembler,
-}
-
-%Assembler = (
-  template => [ qw( $sub_start $init_errs $init_output
+  template => [ qw( $source_file $sub_start $init_errs $init_output
 		    $init_args @perl $return_output $sub_end ) ],
 
+  # Source file and line number
+  source_file => sub {
+    my $m = shift;
+    my @caller; 
+    my $call_level ;
+    do { @caller = caller( ++ $call_level ) }
+	while ( $caller[0] =~ /^Text::MicroMason/ or $m->isa($caller[0]) );
+    my $template_source = ( $caller[1] || 'unknown' ) . " template";
+    join(' ', '#', 'line', 1, '"' .  $template_source . '"' . "\n" )
+  },
+
+  # Subroutine scafolding
   sub_start  => 'sub { ',
   sub_end  => '}',
-
   init_errs => 
     'local $SIG{__DIE__} = sub { die "MicroMason execution failed: ", @_ };',
-
+  
   # Argument processing elements
   init_args => 'my %ARGS = @_ if ($#_ % 2);',
-
+  
   # Output generation
-  init_output => 'my @OUT; my $_out = sub { push @OUT, @_ };',
-  add_output => '  push @OUT, ',
+  init_output => sub { my $m = shift; my $sub = $m->{output_sub} ? '$m->{output_sub}' : 'sub {push @OUT, @_}'; 'my @OUT; my $_out = ' . $sub . ';' },
+  add_output => sub { my $m = shift; $m->{output_sub} ? '&$_out' : 'push @OUT,' },
   return_output => 'join("", @OUT)',
 
   # Mapping between token types
   text_token => 'perl OUT( QUOTED );',
   expr_token => 'perl OUT( do{ TOKEN } );',
   file_token => 'perl OUT( $m->execute( file => do { TOKEN } ) );',
-);
+}
+
+sub assembler_vars {
+  my $self = shift;
+  my %assembler = $self->assembler_rules();
+  
+  my @assembly = @{ delete $assembler{ template } };
+  
+  my %token_map = map { ( /^(.*?)_token$/ )[0] => delete $assembler{$_} } 
+					    grep { /_token$/ } keys %assembler;
+
+  my %fragments = map { $_ => map { ref($_) ? &{$_}( $self ) : $_ } $assembler{$_} } keys %assembler;
+
+  return( \@assembly, \%fragments, \%token_map );
+}
 
 # $perl_code = $mason->assemble( @tokens );
 sub assemble {
   my $self = shift;
   my @tokens = @_;
   
-  my %assembler = $self->assembler_rules();
-  my @assembly = @{ $assembler{ template } };
-  
-  my %token_streams = map { $_ => [] } map { ( /^\W?\@(\w+)$/ ) } @assembly;
-  my %token_map = map { ( /^(.*?)_token$/ )[0] => $assembler{$_} } 
-					    grep { /_token$/ } keys %assembler;
+  my ( $order, $fragments, $token_map ) = $self->assembler_vars();
   
+  my %token_streams = map { $_ => [] } map { ( /^\W?\@(\w+)$/ ) } @$order;
+  my ($output) = $fragments->{add_output};
+
   while ( scalar @tokens ) {
-    my $type = shift @tokens;
-    my $token = shift @tokens;
-    my @functions;
+    my ( $type, $token ) = splice( @tokens, 0, 2 );
     
-    unless ( $token_streams{$type} or $token_map{$type} ) {
+    unless ( $token_streams{$type} or $token_map->{$type} ) {
       my $method = "assemble_$type";
       my $sub = $self->can( $method ) 
 	or $self->croak_msg( "Unexpected token type '$type': '$token'" );
       ($type, $token) = &$sub( $self, $token );
     }
 
-    if ( my $typedef = $token_map{ $type } ) {
+    if ( my $typedef = $token_map->{ $type } ) {
       $typedef =~ s{\bTOKEN\b}{$token}g;
       $typedef =~ s{\bQUOTED\b}{qq(\Q$token\E)}g;
-      $typedef =~ s{\bOUT\b}{$assembler{add_output}}g;
+      $typedef =~ s{\bOUT\b}{$output}g;
       ( $type, $token ) = split ' ', $typedef, 2;
     }
     
@@ -123,12 +265,11 @@ sub assemble {
     
     push @$ary, $token
   }
-    
-  join(' ', '#', 'line', 1, '"' . ( ( caller(2) )[1] || 'unknown' ) . ' template near"' . "\n" ) .
+  
   join( "\n",  map { 
     /^(\W+)(\w+)$/ or $self->croak_msg("Can't assemble $_");
     if ( $1 eq '$' ) {
-      $assembler{ $2 }
+      $fragments->{ $2 }
     } elsif ( $1 eq '@' ) {
       @{ $token_streams{ $2 } }
     } elsif ( $1 eq '!@' ) {
@@ -138,96 +279,13 @@ sub assemble {
     } else {
       $self->croak_msg("Can't assemble $_");
     }
-  } @assembly );
+  } @$order );
 }
 
 ######################################################################
 
-# $code_ref = $mason->eval_sub( $perl_code );
-sub eval_sub {
-  my ( $m, $code ) = @_;
-  package Text::MicroMason::Commands; 
-  eval( $code )
-}
-
 ######################################################################
 
-# ( $type, $data ) = $mason->resolve( $type, $data );
-sub resolve {
-  my ( $self, $src_type, $src_data ) = @_;
-  if ( $src_type eq 'lines' ) {
-    'text' => join "\n", @$src_data
-  } else {
-    $src_type, $src_data 
-  }
-}
-
-# $template = $mason->read_text( $template );
-sub read_text {
-  ref($_[1]) ? $$_[1] : $_[1];
-}
-
-# ( $contents, %path_info ) = $mason->read_file( $filename );
-sub read_file {
-  my ( $self, $file ) = @_;
-  $self->{debug} and $self->debug_msg( "MicroMason reading file:", $file );
-  
-  local *FILE;
-  open FILE, "$file" or $self->croak_msg("MicroMason can't open $file: $!");
-  local $/ = undef;
-  $self->debug_msg("MicroMason reading from '$file'");
-  local $_ = <FILE>;
-  close FILE or $self->croak_msg("MicroMason can't close $file: $!");;
-  return ( $_, source_file => $file );
-}
-
-######################################################################
-
-# $code_ref = $mason->compile( text => $template, %options );
-# $code_ref = $mason->compile( file => $filename, %options );
-sub compile {
-  my ( $self, $src_type, $src_data, %options ) = @_;
-  $self = $self->new( %options ) if ( scalar keys %options );
-  
-  ( $src_type, $src_data, %options ) = $self->resolve( $src_type, $src_data );
-  $self->{debug} and $self->debug_msg("MicroMason read:", $src_type, $src_data); 
-  $self = $self->new( %options ) if ( scalar keys %options );
-  
-  my $src_method = "read_$src_type";
-  ( my $template, %options ) = $self->$src_method( $src_data );
-  $self->{debug} and $self->debug_msg( "MicroMason source:", $template ); 
-  $self = $self->new( %options ) if ( scalar keys %options );
-
-  my @tokens = $self->lex( $template, $options{source_file} );
-  $self->{debug} and $self->debug_msg( "MicroMason tokens:", @tokens ); 
-
-  my $code = $self->assemble( @tokens );
-  $self->{debug} and $self->debug_msg( "MicroMason subdef: $code" );
-  
-  $self->eval_sub( $code ) 
-    or $self->croak_msg( "MicroMason compilation failed:\n$code\n$@\n" )
-}
-
-######################################################################
-
-# $result = $mason->execute( text => $template, @arguments );
-# $result = $mason->execute( file => $filename, @arguments );
-# $result = $mason->execute( text => $template, \%options, @arguments );
-# $result = $mason->execute( file => $filename, \%options, @arguments );
-sub execute {
-  my $self = shift;
-  my $sub = ( $_[0] eq 'code' ) ? do { shift; shift } : 
-	$self->compile( shift, shift, ref($_[0]) ? %{ shift() } : () )
-    or $self->croak_msg("Couldn't compile: $@");
-  &$sub( @_ );
-}
-
-######################################################################
-
-sub debug_msg {
-  (shift)->{debug} and warn( ( ( @_ == 1 ) ? $_[0] : join( ' ', map _printable(), @_ ) ) . "\n")
-}
-
 sub croak_msg {
   local $Carp::CarpLevel = 2;
   shift and Carp::croak( ( @_ == 1 ) ? $_[0] : join(' ', map _printable(), @_) )
@@ -243,75 +301,7 @@ sub _printable {
   local $_ = scalar(@_) ? (shift) : $_;
   return "(undef)" unless defined;
   s/([\r\n\t\"\\\x00-\x1f\x7F-\xFF])/\\$Escape{$1}/sgo;
-  /[^\w\d\-\:\.\'\ ]/ ? "q($_)" : $_;
-}
-
-######################################################################
-
-sub class {
-  my ( $baseclass, @mixins ) = @_;
-  return $baseclass if ( ! @mixins );
-  my $basespace = ( $baseclass =~ m/^(.*\:\:)[^\:]+$/ )[0];
-    
-  my (@names, @packages);
-  foreach my $mixin ( @mixins ) {
-    push @packages, ( $mixin =~ /::/ ) ? $mixin : "$basespace$mixin";
-    
-    my $t_name = $mixin;
-    $t_name =~ s/^$basespace//;
-    $t_name =~ s/\:/_/g;
-    push @names, $t_name;
-  }
-
-  my $name = join('_', @names);
-  
-  my $new_class = $baseclass . "::" . $name;
-  
-  no strict;
-  if ( ! @{ $new_class . "::ISA" } ) {
-    foreach my $mixin ( @packages ) {
-      my $t_file = "$mixin.pm";
-      $t_file =~ s{::}{/}g;
-      unless ( $INC{ $t_file } ) {
-	# warn "require $t_file";
-	require $t_file
-      }
-    }
-    @{ $new_class . "::ISA" } = ( reverse(@packages), $baseclass );
-    # warn "-> $new_class ISA ". join(' ', @{ $new_class . "::ISA" }) ."\n";
-  }
-  
-  return $new_class;
-}
-
-######################################################################
-
-sub NEXT {
-  my ( $self, $method, @args ) = @_;
-  
-  my ( $filename, $subname );
-  my $depth = 1;
-  do { ( $filename, $subname ) = ( caller( $depth ++ ) )[1,3] }
-    while ($filename eq '(eval)' || $subname eq '(eval)');
-  $subname =~ s/.*\:\://;
-  my $package = ( caller($depth - 2) )[0];
-  warn "-> $method - $depth - $subname $package\n" if ( $method ne $subname );
-  
-  my @classes = ref($self) || $self;
-  my @isa;
-  while ( my $class = shift @classes ) {
-    push @isa, $class;
-    no strict;
-    unshift @classes, @{ $class . "::ISA" };
-  }
-  while ( my $class = shift @isa ) {
-    last if ( $class eq $package )
-  }
-  while ( my $class = shift @isa ) {
-    next unless my $sub = $class->can( $method );
-    return &$sub( $self, @args );
-  }
-  $self->croak_msg( "Can't find NEXT method" );
+  /[^\w\d\-\:\.\']/ ? "q($_)" : $_;
 }
 
 ######################################################################
@@ -324,12 +314,12 @@ __END__
 
 =head1 NAME
 
-Text::MicroMason::Base - Abstract Compiler for Simple Templating 
+Text::MicroMason::Base - Abstract Template Compiler 
 
 
 =head1 SYNOPSIS
 
-Create a Mason object to interpret the templates:
+Create a MicroMason object to interpret the templates:
 
     use Text::MicroMason;
     my $mason = Text::MicroMason->new();
@@ -351,25 +341,20 @@ Templates stored in files can be run directly or included in others:
 
 =head1 DESCRIPTION
 
-The Text::MicroMason::Base class provides a parser and execution environment for a simple templating system based on HTML::Mason.
-
+The Text::MicroMason::Base class provides a parser and execution environment for an extensible templating system.
 
 =head2 Public Methods
 
 =over 4
 
-=item class()
-
-  $class = Text::MicroMason::Base->class( @Mixins );
-
-Creates a subclass of this package that also inherits from the other classes named.
-
 =item new()
 
-  $mason = $class->new( %options );
-  $clone = $mason->new( %options );
+  $mason = Text::MicroMason::Base->new( -Mixin1, -Mixin2, %attribs );
 
-Creates a new instance with the provided key value pairs.
+Creates a new Text::MicroMason object with mixins and attributes. 
+
+Arguments beginning with a dash will be added as mixin classes.
+Other arguments are added to the hash of attributes.
 
 =item compile()
 
@@ -390,21 +375,90 @@ Returns the results produced by the template, given the provided arguments.
 
 =back
 
+=head2 Attributes
+
+Attributes can be set in a call to new() and locally overridden in a call to compile().
+
+=over 4
+
+=item output_sub
+
+Optional reference to a subroutine to call with each piece of template output. If this is enabled, template subroutines will return an empty string. 
+
+=back
+
 =head2 Private Methods
 
 The following internal methods are used to implement the public interface described above, and may be overridden by subclasses and mixins.
 
 =over 4
 
-=item defaults
+=item class()
+
+  $class = Text::MicroMason::Base->class( @Mixins );
+
+Creates a subclass of this package that also inherits from the other classes named. Provided by Class::MixinFactory::HasAFactory. 
+
+=item create()
+
+  $mason = $class->create( %options );
+  $clone = $mason->create( %options );
+
+Creates a new instance with the provided key value pairs.
+
+To obtain the functionality of one of the supported mixin classes, use the class method to generate the mixed class before calling create(), as is done by new().
+
+=item defaults()
 
 This class method is called by new() to provide key-value pairs to be included in the new instance.
 
+=item prepare()
+
+  ($self, $src_type, $src_data) = $self->prepare($src_type, $src_data, %options)
+
+Called by compile(), the prepare method allows for single-use attributes and provides a hook for mixin functionality. 
+
+The prepare method provides a hook for mixins to normalize or resolve the template source type and value arguments in various ways before the template is read using one of the read_type() methods. 
+
+It returns an object reference that may be a clone of the original mason object with various compile-time attributes applied. The cloning is a shallow copy performed by the create() method. This means that the $m object visible to a template may not be the same as the MicroMason object on which compile() was originally called.
+
+Please note that this clone-on-prepare behavior is subject to change in future releases.
+
+=item interpret
+
+   $perl_code = $mason->interpret( $src_type, $src_data );
+
+Called by compile(), the interpret method then calls the read(), lex(), and assemble() methods.
+
+=item read
+
+  $template = $mason->read( $src_type, $src_data );
+
+Called by interpret(). Calls one of the below read_* methods.
+
+=item read_text
+
+  $template = $mason->read_text( $template );
+
+Called by read() when the template source type is "text", this method simply returns the value of the text string passed to it. 
+
+=item read_file
+
+  ( $contents, %path_info ) = $mason->read_file( $filename );
+
+Called by read() when the template source type is "file", this method reads and returns the contents of the named file.
+
+=item read_handle
+
+  $template = $mason->read_handle( $filehandle );
+
+Called by read() when the template source type is "handle", this method reads and returns the contents of the filehandle passed to it. 
+
 =item lex
 
   @token_pairs = $mason->lex( $template );
 
-Parses the source text and returns a list of pairs of token types and values. Loops through repeated calls to lex_token().
+Called by interpret(). Parses the source text and returns a list of pairs of token types and values. Loops through repeated calls to lex_token().
 
 =item lex_token
 
@@ -418,7 +472,7 @@ Abstract method; must be implemented by subclasses.
 
   $perl_code = $mason->assemble( @tokens );
 
-Assembles the parsed token series into the source code for the equivalent Perl subroutine.
+Called by interpret(). Assembles the parsed token series into the source code for the equivalent Perl subroutine.
 
 =item assembler_rules()
 
@@ -430,29 +484,7 @@ The assembly template defines the types of blocks supported and the order they a
 
   $code_ref = $mason->eval_sub( $perl_code );
 
-Compiles the Perl source code for a template using eval(), and returns a code reference. 
-
-=item resolve
-
-  ( $type, $data ) = $mason->resolve( $type, $data );
-
-Called by compile(), the resolve method allows the template source type and value arguments to be normalized or resolved in various ways before the template is read using one of the read_type() methods. 
-
-=item read_text
-
-  $template = $mason->read_text( $template );
-
-Called by compile() when the template source type is "text", this method simply returns the value of the text string passed to it. 
-
-=item read_file
-
-  ( $contents, %path_info ) = $mason->read_file( $filename );
-
-Called by compile() when the template source type is "file", this method reads and returns the contents of the named file.
-
-=item debug_msg
-
-Called to provide a debugging message for developer reference. No output is produced unless the object's 'debug' flag is true.
+Called by compile(). Compiles the Perl source code for a template using eval(), and returns a code reference. 
 
 =item croak_msg 
 
@@ -460,9 +492,7 @@ Called when a fatal exception has occured.
 
 =item NEXT
 
-Enhanced superclass method dispatch for use inside mixin class methods. Allows mixin classes to redispatch to other classes in the inheritance tree without themselves inheriting from anything. 
-
-(This is similar to the functionality provided by NEXT::ACTUAL, but without using AUTOLOAD; for a more generalized approach to this issue see L<NEXT>.)
+Enhanced superclass method dispatch for use inside mixin class methods. Allows mixin classes to redispatch to other classes in the inheritance tree without themselves inheriting from anything. Provided by Class::MixinFactory::NEXT. 
 
 =back
 
@@ -478,16 +508,6 @@ Converts non-printable characters to readable form using the standard backslash
 
 =back
 
-=head2 Supported Attributes
-
-=over 4
-
-=item debug
-
-Boolean value. Debugging flag activates warns throughout the code. Used by debug_msg(). Defaults to 0.
-
-=back
-
 =head1 EXTENDING
 
 You can add functionality to this module by creating subclasses or mixin classes. 
@@ -498,9 +518,9 @@ Key areas for subclass writers are:
 
 =over 4
 
-=item resolve
+=item prepare
 
-You can intercept and re-write template source arguments by implementing this method.
+You can intercept and re-write template source arguments by overriding this method.
 
 =item read_*
 
